=== modified file 'NEWS'
--- NEWS	2009-12-14 06:05:30 +0000
+++ NEWS	2009-12-14 09:31:20 +0000
@@ -26,6 +26,9 @@
 * ``bzr commit`` now has a ``--commit-time`` option.
   (Alexander Sack, #459276)
 
+* New option ``bzr unshelve --keep`` applies the changes and leaves them
+  on the shelf.  (Martin Pool, Oscar Fuentes, #492091)
+
 * The ``BZR_COLUMNS`` envrionment variable can be set to force bzr to
   respect a given terminal width. This can be useful when output is
   redirected or in obscure cases where the default value is not

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2009-12-09 08:38:02 +0000
+++ bzrlib/builtins.py	2009-12-11 05:48:21 +0000
@@ -5793,7 +5793,8 @@
             enum_switch=False, value_switches=True,
             apply="Apply changes and remove from the shelf.",
             dry_run="Show changes, but do not apply or remove them.",
-            delete_only="Delete changes without applying them."
+            delete_only="Delete changes without applying them.",
+            keep="Apply changes but don't delete them.",
         )
     ]
     _see_also = ['shelve']

=== modified file 'bzrlib/shelf_ui.py'
--- bzrlib/shelf_ui.py	2009-11-26 17:06:32 +0000
+++ bzrlib/shelf_ui.py	2009-12-11 05:48:21 +0000
@@ -414,9 +414,12 @@
             if action == 'dry-run':
                 apply_changes = False
                 delete_shelf = False
-            if action == 'delete-only':
+            elif action == 'delete-only':
                 apply_changes = False
                 read_shelf = False
+            elif action == 'keep':
+                apply_changes = True
+                delete_shelf = False
         except:
             tree.unlock()
             raise

=== modified file 'bzrlib/tests/blackbox/test_shelve.py'
--- bzrlib/tests/blackbox/test_shelve.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_shelve.py	2009-12-11 05:48:21 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 
 from bzrlib import shelf
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.script import ScriptRunner
+
 
 class TestShelveList(TestCaseWithTransport):
 
@@ -65,6 +67,27 @@
         self.failIfExists('file')
         self.assertIs(None, tree.get_shelf_manager().last_shelf())
 
+    def test_unshelve_keep(self):
+        # https://bugs.edge.launchpad.net/bzr/+bug/492091
+        tree = self.make_branch_and_tree('.')
+        # shelve apparently unhappy working with a tree with no root yet
+        tree.commit('make root')
+        self.build_tree(['file'])
+
+        sr = ScriptRunner()
+        sr.run_script(self, '''
+$ bzr add file
+$ bzr shelve --all -m Foo
+$ bzr shelve --list
+  1: Foo
+$ bzr unshelve --keep
+$ bzr shelve --list
+  1: Foo
+$ cat file
+contents of file
+''')
+
+
 
 class TestShelveRelpath(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/script.py'
--- bzrlib/tests/script.py	2009-11-08 20:28:36 +0000
+++ bzrlib/tests/script.py	2009-12-11 05:29:35 +0000
@@ -107,9 +107,11 @@
                 error = []
             error.append(line[2:] + '\n')
         else:
+            # can happen if the first line is not recognized as a command, eg
+            # if the prompt has leading whitespace
             if output is None:
                 if cmd_cur is None:
-                    raise SyntaxError('No command for that output',
+                    raise SyntaxError('No command for line %r' % (line,),
                                       (file_name, lineno, 1, orig))
                 output = []
             output.append(line + '\n')

